@@ -348,12 +348,12 @@ class TestMedium(unittest.TestCase):
         sim = mp.Simulation(cell_size=cell_size, resolution=resolution, sources=valid_sources[0],
                             extra_materials=[mat])
         fregion = mp.FluxRegion(center=mp.Vector3(0, 1), size=mp.Vector3(2, 2), direction=mp.X)
-        sim.add_flux(15, 6, 2, fregion)
+        sim.add_flux(18, 6, 2, fregion)
         check_warnings(sim)
 
         # Invalid geometry material
         sim = mp.Simulation(cell_size=cell_size, resolution=resolution, sources=valid_sources[0], geometry=geom)
-        sim.add_flux(15, 6, 2, fregion)
+        sim.add_flux(18, 6, 2, fregion)
         check_warnings(sim)
 
     def test_transform(self):
@@ -601,8 +601,8 @@ class TestMatrix(unittest.TestCase):
     def test_to_numpy_array(self):
         m = mp.Matrix(mp.Vector3(1+1j), mp.Vector3(1+1j), mp.Vector3(1+1j))
         adjoint = m.H
-        m_arr = np.matrix(m)
-        np_adjoint = m_arr.H
+        m_arr = np.array(m)
+        np_adjoint = m_arr.conj().transpose()
         np.testing.assert_allclose(adjoint, np_adjoint)
 
 
