@@ -1,6 +1,5 @@
+# -*- coding: utf-8 -*-
 from collections import namedtuple
-from collections import OrderedDict
-from collections import Sequence
 import warnings
 
 import numpy as np
@@ -46,7 +45,9 @@ default_field_parameters = {
 default_eps_parameters = {
         'interpolation':'spline36',
         'cmap':'binary',
-        'alpha':1.0
+        'alpha':1.0,
+        'contour':False,
+        'contour_linewidth':1
     }
 
 default_boundary_parameters = {
@@ -303,7 +304,7 @@ def plot_volume(sim,ax,volume,output_plane=None,plotting_parameters=None,label=N
             return ax
     return ax
 
-def plot_eps(sim,ax,output_plane=None,eps_parameters=None,omega=0):
+def plot_eps(sim,ax,output_plane=None,eps_parameters=None,frequency=0):
     if sim.structure is None:
         sim.init_sim()
 
@@ -342,9 +343,12 @@ def plot_eps(sim,ax,output_plane=None,eps_parameters=None,omega=0):
     else:
         raise ValueError("A 2D plane has not been specified...")
 
-    eps_data = np.rot90(np.real(sim.get_array(center=center, size=cell_size, component=mp.Dielectric, omega=omega)))
+    eps_data = np.rot90(np.real(sim.get_array(center=center, size=cell_size, component=mp.Dielectric, frequency=frequency)))
     if mp.am_master():
-        ax.imshow(eps_data, extent=extent, **eps_parameters)
+        if eps_parameters['contour']:
+            ax.contour(eps_data, 0, colors='black', origin='upper', extent=extent, linewidths=eps_parameters['contour_linewidth'])
+        else:
+            ax.imshow(eps_data, extent=extent, **filter_dict(eps_parameters, ax.imshow))
         ax.set_xlabel(xlabel)
         ax.set_ylabel(ylabel)
 
@@ -512,9 +516,11 @@ def plot_fields(sim,ax=None,fields=None,output_plane=None,field_parameters=None)
     return ax
 
 def plot2D(sim,ax=None, output_plane=None, fields=None, labels=False,
-            eps_parameters=None,boundary_parameters=None,
-            source_parameters=None,monitor_parameters=None,
-            field_parameters=None, omega=None):
+           eps_parameters=None,boundary_parameters=None,
+           source_parameters=None,monitor_parameters=None,
+           field_parameters=None, frequency=None,
+           plot_eps_flag=True, plot_sources_flag=True,
+           plot_monitors_flag=True, plot_boundaries_flag=True):
 
     # Initialize the simulation
     if sim.structure is None:
@@ -524,21 +530,21 @@ def plot2D(sim,ax=None, output_plane=None, fields=None, labels=False,
         from matplotlib import pyplot as plt
         ax = plt.gca()
     # Determine a frequency to plot all epsilon
-    if omega is None:
+    if frequency is None:
         try:
             # Continuous sources
-            omega = sim.sources[0].frequency
+            frequency = sim.sources[0].frequency
         except:
             try:
                 # Gaussian sources
-                omega = sim.sources[0].src.frequency
+                frequency = sim.sources[0].src.frequency
             except:
                 try:
                     # Custom sources
-                    omega = sim.sources[0].src.center_frequency
+                    frequency = sim.sources[0].src.center_frequency
                 except:
                     # No sources
-                    omega = 0
+                    frequency = 0
 
     # validate the output plane to ensure proper 2D coordinates
     from meep.simulation import Volume
@@ -546,16 +552,20 @@ def plot2D(sim,ax=None, output_plane=None, fields=None, labels=False,
     output_plane = Volume(center=sim_center,size=sim_size)
 
     # Plot geometry
-    ax = plot_eps(sim,ax,output_plane=output_plane,eps_parameters=eps_parameters,omega=omega)
+    if plot_eps_flag:
+        ax = plot_eps(sim,ax,output_plane=output_plane,eps_parameters=eps_parameters,frequency=frequency)
 
     # Plot boundaries
-    ax = plot_boundaries(sim,ax,output_plane=output_plane,boundary_parameters=boundary_parameters)
+    if plot_boundaries_flag:
+        ax = plot_boundaries(sim,ax,output_plane=output_plane,boundary_parameters=boundary_parameters)
 
     # Plot sources
-    ax = plot_sources(sim,ax,output_plane=output_plane,labels=labels,source_parameters=source_parameters)
+    if plot_sources_flag:
+        ax = plot_sources(sim,ax,output_plane=output_plane,labels=labels,source_parameters=source_parameters)
 
     # Plot monitors
-    ax = plot_monitors(sim,ax,output_plane=output_plane,labels=labels,monitor_parameters=monitor_parameters)
+    if plot_monitors_flag:
+        ax = plot_monitors(sim,ax,output_plane=output_plane,labels=labels,monitor_parameters=monitor_parameters)
 
     # Plot fields
     ax = plot_fields(sim,ax,fields,output_plane=output_plane,field_parameters=field_parameters)
@@ -696,8 +706,74 @@ def visualize_chunks(sim):
 #                       to pass to plot2D()
 
 class Animate2D(object):
-    def __init__(self,sim,fields,f=None,realtime=False,normalize=False,
-    plot_modifiers=None,**customization_args):
+    """
+    A class used to record the fields during timestepping (i.e., a [`run`](#run-functions)
+    function). The object is initialized prior to timestepping by specifying the
+    simulation object and the field component. The object can then be passed to any
+    [step-function modifier](#step-function-modifiers). For example, one can record the
+    E<sub>z</sub> fields at every one time unit using:
+
+    ```py
+    animate = mp.Animate2D(sim,
+                           fields=mp.Ez,
+                           realtime=True,
+                           field_parameters={'alpha':0.8, 'cmap':'RdBu', 'interpolation':'none'},
+                           boundary_parameters={'hatch':'o', 'linewidth':1.5, 'facecolor':'y', 'edgecolor':'b', 'alpha':0.3})
+
+    sim.run(mp.at_every(1,animate),until=25)
+    ```
+
+    By default, the object saves each frame as a PNG image into memory (not disk). This is
+    typically more memory efficient than storing the actual fields. If the user sets the
+    `normalize` argument, then the object will save the actual field information as a
+    NumPy array to be normalized for post processing. The fields of a figure can also be
+    updated in realtime by setting the `realtime` flag. This does not work for
+    IPython/Jupyter notebooks, however.
+
+    Once the simulation is run, the animation can be output as an interactive JSHTML
+    object, an mp4, or a GIF.
+
+    Multiple `Animate2D` objects can be initialized and passed to the run function to
+    track different volume locations (using `mp.in_volume`) or field components.
+    """
+    def __init__(self, sim, fields, f=None, realtime=False, normalize=False,
+                 plot_modifiers=None, **customization_args):
+        """
+        Construct an `Animate2D` object.
+
+        + **`sim`** — Simulation object.
+
+        + **`fields`** — Field component to record at each time instant.
+
+        + **`f=None`** — Optional `matplotlib` figure object that the routine will update
+          on each call. If not supplied, then a new one will be created upon
+          initialization.
+
+        + **`realtime=False`** — Whether or not to update a figure window in realtime as
+          the simulation progresses. Disabled by default. Not compatible with
+          IPython/Jupyter notebooks.
+
+        + **`normalize=False`** — Records fields at each time step in memory in a NumPy
+          array and then normalizes the result by dividing by the maximum field value at a
+          single point in the cell over all the time snapshots.
+
+        + **`plot_modifiers=None`** — A list of functions that can modify the figure's
+          `axis` object. Each function modifier accepts a single argument, an `axis`
+          object, and must return that same axis object. The following modifier changes
+          the `xlabel`:
+
+        ```py
+          def mod1(ax):
+              ax.set_xlabel('Testing')
+              return ax
+
+          plot_modifiers = [mod1]
+        ```
+
+        + **`**customization_args`** — Customization keyword arguments passed to
+          `plot2D()` (i.e. `labels`, `eps_parameters`, `boundary_parameters`, etc.)
+        """
+
         self.fields = fields
 
         if f:
@@ -773,7 +849,7 @@ class Animate2D(object):
 
             # Normalize the frames, if requested, and export
             if self.normalize and mp.am_master():
-                if mp.cvar.verbosity > 0:
+                if mp.verbosity.meep > 0:
                     print("Normalizing field data...")
                 fields = np.array(self.cumulative_fields) / np.max(np.abs(self.cumulative_fields),axis=(0,1,2))
                 for k in range(len(self.cumulative_fields)):
@@ -810,6 +886,11 @@ class Animate2D(object):
             for i, frame_data in enumerate(frame_list))
 
     def to_jshtml(self,fps):
+        """
+        Outputs an interactable JSHTML animation object that is embeddable in Jupyter
+        notebooks. The object is packaged with controls to manipulate the video's
+        playback. User must specify a frame rate `fps` in frames per second.
+        """
         # Exports a javascript enabled html object that is
         # ready for jupyter notebook embedding.
         # modified from matplotlib/animation.py code.
@@ -847,6 +928,13 @@ class Animate2D(object):
             return JS_Animation(html_string)
 
     def to_gif(self,fps,filename):
+        """
+        Generates and outputs a GIF file of the animation with the filename, `filename`,
+        and the frame rate, `fps`. Note that GIFs are significantly larger than mp4 videos
+        since they don't use any compression. Artifacts are also common because the GIF
+        format only supports 256 colors from a _predefined_ color palette. Requires
+        `ffmpeg`.
+        """
         # Exports a gif of the recorded animation
         # requires ffmpeg to be installed
         # modified from the matplotlib library
@@ -866,7 +954,7 @@ class Animate2D(object):
                         '-vf', 'pad=width=ceil(iw/2)*2:height=ceil(ih/2)*2',
                         '-an', filename  # output filename
             ]
-            if mp.cvar.verbosity > 0:
+            if mp.verbosity.meep > 0:
                 print("Generating GIF...")
             proc = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
             for i in range(len(self._saved_frames)):
@@ -877,6 +965,11 @@ class Animate2D(object):
         return
 
     def to_mp4(self,fps,filename):
+        """
+        Generates and outputs an mp4 video file of the animation with the filename,
+        `filename`, and the frame rate, `fps`. Default encoding is h264 with yuv420p
+        format. Requires `ffmpeg`.
+        """
         # Exports an mp4 of the recorded animation
         # requires ffmpeg to be installed
         # modified from the matplotlib library
@@ -898,7 +991,7 @@ class Animate2D(object):
                         '-vf', 'pad=width=ceil(iw/2)*2:height=ceil(ih/2)*2',
                         '-an', filename  # output filename
             ]
-            if mp.cvar.verbosity > 0:
+            if mp.verbosity.meep > 0:
                 print("Generating MP4...")
             proc = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
             for i in range(len(self._saved_frames)):
