@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2019 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2021 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ typedef struct {
   cdouble *fields;
   ptrdiff_t *offsets;
 
-  double omega;
+  double frequency;
 
   int ninveps;
   component inveps_cs[3];
@@ -76,6 +76,8 @@ typedef struct {
   component invmu_cs[3];
   direction invmu_ds[3];
 
+  bool snap;
+  bool empty_dim[5];
 } array_slice_data;
 
 #define UNUSED(x) (void)x // silence compiler warnings
@@ -282,7 +284,7 @@ static void get_array_slice_chunkloop(fields_chunk *fc, int ichnk, component cgr
 
   ptrdiff_t *off = data->offsets;
   component *cS = data->cS;
-  double omega = data->omega;
+  double frequency = data->frequency;
   complex<double> *fields = data->fields, *ph = data->ph;
   const component *iecs = data->inveps_cs;
   const direction *ieds = data->inveps_ds;
@@ -294,7 +296,7 @@ static void get_array_slice_chunkloop(fields_chunk *fc, int ichnk, component cgr
 
   for (int i = 0; i < num_components; ++i) {
     cS[i] = S.transform(data->components[i], -sn);
-    if (cS[i] == Dielectric || cS[i] == Permeability)
+    if (cS[i] == Dielectric || cS[i] == Permeability || cS[i] == NO_COMPONENT)
       ph[i] = 1.0;
     else {
       fc->gv.yee2cent_offsets(cS[i], off[2 * i], off[2 * i + 1]);
@@ -306,6 +308,22 @@ static void get_array_slice_chunkloop(fields_chunk *fc, int ichnk, component cgr
   for (int k = 0; k < data->ninvmu; ++k)
     fc->gv.yee2cent_offsets(imcs[k], imos[2 * k], imos[2 * k + 1]);
 
+  /*****************************************************************/
+  /* For collapsing empty dimensions, we want to retain interpolation
+     weights for empty dimensions, but not interpolation weights for
+     integration of edge pixels (for retain_interp_weights == true).
+     All of the weights are stored in (s0, s1, e0, e1), so we make
+     a copy of these with the weights for non-empty dimensions set to 1. */
+  vec s0i(s0), s1i(s1), e0i(e0), e1i(e1);
+  LOOP_OVER_DIRECTIONS(fc->gv.dim, d) {
+    if (!data->empty_dim[d]) {
+      s0i.set_direction(d, 1.0);
+      s1i.set_direction(d, 1.0);
+      e0i.set_direction(d, 1.0);
+      e1i.set_direction(d, 1.0);
+    }
+  }
+
   vec rshift(shift * (0.5 * fc->gv.inva));
   // main loop over all grid points owned by this field chunk.
   LOOP_OVER_IVECS(fc->gv, is, ie, idx) {
@@ -323,28 +341,28 @@ static void get_array_slice_chunkloop(fields_chunk *fc, int ichnk, component cgr
         fields[i] = IVEC_LOOP_WEIGHT(s0, s1, e0, e1, dV0 + dV1 * loop_i2);
       }
       else if (cS[i] == Dielectric) {
-        double tr = 0.0;
+        complex<double> tr(0.0, 0.0);
         for (int k = 0; k < data->ninveps; ++k) {
-          tr += (fc->s->get_chi1inv_at_pt(iecs[k], ieds[k], idx, omega) +
-                 fc->s->get_chi1inv_at_pt(iecs[k], ieds[k], idx + ieos[2 * k], omega) +
-                 fc->s->get_chi1inv_at_pt(iecs[k], ieds[k], idx + ieos[1 + 2 * k], omega) +
+          tr += (fc->s->get_chi1inv_at_pt(iecs[k], ieds[k], idx, frequency) +
+                 fc->s->get_chi1inv_at_pt(iecs[k], ieds[k], idx + ieos[2 * k], frequency) +
+                 fc->s->get_chi1inv_at_pt(iecs[k], ieds[k], idx + ieos[1 + 2 * k], frequency) +
                  fc->s->get_chi1inv_at_pt(iecs[k], ieds[k], idx + ieos[2 * k] + ieos[1 + 2 * k],
-                                          omega));
-          if (tr == 0.0) tr += 4.0; // default inveps == 1
+                                          frequency));
+          if (abs(tr) == 0.0) tr += 4.0; // default inveps == 1
         }
-        fields[i] = (4 * data->ninveps) / tr;
+        fields[i] = IVEC_LOOP_WEIGHT(s0i, s1i, e0i, e1i, 1.0) * (4.0 * data->ninveps) / tr;
       }
       else if (cS[i] == Permeability) {
-        double tr = 0.0;
+        complex<double> tr(0.0, 0.0);
         for (int k = 0; k < data->ninvmu; ++k) {
-          tr += (fc->s->get_chi1inv_at_pt(imcs[k], imds[k], idx, omega) +
-                 fc->s->get_chi1inv_at_pt(imcs[k], imds[k], idx + imos[2 * k], omega) +
-                 fc->s->get_chi1inv_at_pt(imcs[k], imds[k], idx + imos[1 + 2 * k], omega) +
+          tr += (fc->s->get_chi1inv_at_pt(imcs[k], imds[k], idx, frequency) +
+                 fc->s->get_chi1inv_at_pt(imcs[k], imds[k], idx + imos[2 * k], frequency) +
+                 fc->s->get_chi1inv_at_pt(imcs[k], imds[k], idx + imos[1 + 2 * k], frequency) +
                  fc->s->get_chi1inv_at_pt(imcs[k], imds[k], idx + imos[2 * k] + imos[1 + 2 * k],
-                                          omega));
-          if (tr == 0.0) tr += 4.0; // default invmu == 1
+                                          frequency));
+          if (abs(tr) == 0.0) tr += 4.0; // default invmu == 1
         }
-        fields[i] = (4 * data->ninvmu) / tr;
+        fields[i] = IVEC_LOOP_WEIGHT(s0i, s1i, e0i, e1i, 1.0) * (4.0 * data->ninvmu) / tr;
       }
       else {
         double f[2];
@@ -355,7 +373,7 @@ static void get_array_slice_chunkloop(fields_chunk *fc, int ichnk, component cgr
                            fc->f[cS[i]][k][idx + off[2 * i] + off[2 * i + 1]]);
           else
             f[k] = 0;
-        fields[i] = complex<double>(f[0], f[1]) * ph[i];
+        fields[i] = IVEC_LOOP_WEIGHT(s0i, s1i, e0i, e1i, 1.0) * complex<double>(f[0], f[1]) * ph[i];
       }
     }
 
@@ -410,7 +428,7 @@ cdouble *array_to_all(cdouble *array, size_t array_size) {
 /***************************************************************/
 int fields::get_array_slice_dimensions(const volume &where, size_t dims[3], direction dirs[3],
                                        bool collapse_empty_dimensions, bool snap_empty_dimensions,
-                                       vec *min_max_loc, void *caller_data) {
+                                       vec *min_max_loc, void *caller_data, component cgrid) {
   am_now_working_on(FieldOutput);
 
   // use a local data structure if the caller didn't provide one
@@ -434,9 +452,10 @@ int fields::get_array_slice_dimensions(const volume &where, size_t dims[3], dire
   }
 
   bool use_symmetry = true;
-  loop_in_chunks(get_array_slice_dimensions_chunkloop, (void *)data, where, Centered, use_symmetry,
+  loop_in_chunks(get_array_slice_dimensions_chunkloop, (void *)data, where, cgrid, use_symmetry,
                  snap_empty_dimensions);
 
+  am_now_working_on(MpiTime);
   data->min_corner = -max_to_all(-data->min_corner); // i.e., min_to_all
   data->max_corner = max_to_all(data->max_corner);
   if (min_max_loc) LOOP_OVER_DIRECTIONS(gv.dim, d) {
@@ -444,6 +463,7 @@ int fields::get_array_slice_dimensions(const volume &where, size_t dims[3], dire
       max_loc->set_direction(d, max_to_all(max_loc->in_direction(d)));
     }
   data->num_chunks = sum_to_all(data->num_chunks);
+  finished_working();
   if (data->num_chunks == 0 || !(data->min_corner <= data->max_corner))
     return 0; // no data to write;
 
@@ -468,161 +488,6 @@ int fields::get_array_slice_dimensions(const volume &where, size_t dims[3], dire
   return rank;
 }
 
-/**********************************************************************/
-/* precisely one of fun, rfun, should be non-NULL                     */
-/**********************************************************************/
-void *fields::do_get_array_slice(const volume &where, std::vector<component> components,
-                                 field_function fun, field_rfunction rfun, void *fun_data,
-                                 void *vslice, double omega) {
-  am_now_working_on(FieldOutput);
-
-  /***************************************************************/
-  /* call get_array_slice_dimensions to get slice dimensions and */
-  /* partially initialze an array_slice_data struct              */
-  /***************************************************************/
-  // by tradition, empty dimensions in time-domain field arrays are *not* collapsed
-  // TODO make this a caller-specifiable parameter to get_array_slice()?
-  bool collapse = false, snap = true;
-  size_t dims[3];
-  direction dirs[3];
-  array_slice_data data;
-  get_array_slice_dimensions(where, dims, dirs, collapse, snap, 0, &data);
-  size_t slice_size = data.slice_size;
-
-  bool complex_data = (rfun == 0);
-  cdouble *zslice;
-  double *slice;
-  if (vslice == 0) {
-    if (complex_data) {
-      zslice = new cdouble[slice_size];
-      memset(zslice, 0, slice_size * sizeof(cdouble));
-      vslice = (void *)zslice;
-    }
-    else {
-      slice = new double[slice_size];
-      memset(slice, 0, slice_size * sizeof(double));
-      vslice = (void *)slice;
-    }
-  }
-
-  data.vslice = vslice;
-  data.fun = fun;
-  data.rfun = rfun;
-  data.fun_data = fun_data;
-  data.components = components;
-  data.omega = omega;
-  int num_components = components.size();
-  data.cS = new component[num_components];
-  data.ph = new cdouble[num_components];
-  data.fields = new cdouble[num_components];
-  data.offsets = new ptrdiff_t[2 * num_components];
-  memset(data.offsets, 0, 2 * num_components * sizeof(ptrdiff_t));
-
-  /* compute inverse-epsilon directions for computing Dielectric fields */
-  data.ninveps = 0;
-  bool needs_dielectric = false;
-  for (int i = 0; i < num_components; ++i)
-    if (components[i] == Dielectric) {
-      needs_dielectric = true;
-      break;
-    }
-  if (needs_dielectric) FOR_ELECTRIC_COMPONENTS(c) if (gv.has_field(c)) {
-      if (data.ninveps == 3) abort("more than 3 field components??");
-      data.inveps_cs[data.ninveps] = c;
-      data.inveps_ds[data.ninveps] = component_direction(c);
-      ++data.ninveps;
-    }
-
-  /* compute inverse-mu directions for computing Permeability fields */
-  data.ninvmu = 0;
-  bool needs_permeability = false;
-  for (int i = 0; i < num_components; ++i)
-    if (components[i] == Permeability) {
-      needs_permeability = true;
-      break;
-    }
-  if (needs_permeability) FOR_MAGNETIC_COMPONENTS(c) if (gv.has_field(c)) {
-      if (data.ninvmu == 3) abort("more than 3 field components??");
-      data.invmu_cs[data.ninvmu] = c;
-      data.invmu_ds[data.ninvmu] = component_direction(c);
-      ++data.ninvmu;
-    }
-
-  loop_in_chunks(get_array_slice_chunkloop, (void *)&data, where, Centered, true, true);
-
-  /***************************************************************/
-  /*consolidate full array on all cores                          */
-  /***************************************************************/
-  vslice = (void *)array_to_all((double *)vslice, (complex_data ? 2 : 1) * slice_size);
-
-  delete[] data.offsets;
-  delete[] data.fields;
-  delete[] data.ph;
-  delete[] data.cS;
-  finished_working();
-
-  return vslice;
-}
-
-/***************************************************************/
-/* entry points to get_array_slice                             */
-/***************************************************************/
-double *fields::get_array_slice(const volume &where, std::vector<component> components,
-                                field_rfunction rfun, void *fun_data, double *slice, double omega) {
-  return (double *)do_get_array_slice(where, components, 0, rfun, fun_data, (void *)slice, omega);
-}
-
-cdouble *fields::get_complex_array_slice(const volume &where, std::vector<component> components,
-                                         field_function fun, void *fun_data, cdouble *slice,
-                                         double omega) {
-  return (cdouble *)do_get_array_slice(where, components, fun, 0, fun_data, (void *)slice, omega);
-}
-
-double *fields::get_array_slice(const volume &where, component c, double *slice, double omega) {
-  std::vector<component> components(1);
-  components[0] = c;
-  return (double *)do_get_array_slice(where, components, 0, default_field_rfunc, 0, (void *)slice,
-                                      omega);
-}
-
-double *fields::get_array_slice(const volume &where, derived_component c, double *slice,
-                                double omega) {
-  int nfields;
-  component carray[12];
-  field_rfunction rfun = derived_component_func(c, gv, nfields, carray);
-  std::vector<component> cs(carray, carray + nfields);
-  return (double *)do_get_array_slice(where, cs, 0, rfun, &nfields, (void *)slice, omega);
-}
-
-cdouble *fields::get_complex_array_slice(const volume &where, component c, cdouble *slice,
-                                         double omega) {
-  std::vector<component> components(1);
-  components[0] = c;
-  return (cdouble *)do_get_array_slice(where, components, default_field_func, 0, 0, (void *)slice,
-                                       omega);
-}
-
-cdouble *fields::get_source_slice(const volume &where, component source_slice_component,
-                                  cdouble *slice) {
-
-  size_t dims[3];
-  direction dirs[3];
-  vec min_max_loc[2];
-  bool collapse = false, snap = false;
-  int rank = get_array_slice_dimensions(where, dims, dirs, collapse, snap, min_max_loc);
-  size_t slice_size = dims[0] * (rank >= 2 ? dims[1] : 1) * (rank == 3 ? dims[2] : 1);
-
-  source_slice_data data;
-  data.source_component = source_slice_component;
-  data.slice_imin = gv.round_vec(min_max_loc[0]);
-  data.slice_imax = gv.round_vec(min_max_loc[1]);
-  data.slice = slice ? slice : new cdouble[slice_size];
-  if (!data.slice) abort("%s:%i: out of memory (%zu)", __FILE__, __LINE__, slice_size);
-
-  loop_in_chunks(get_source_slice_chunkloop, (void *)&data, where, Centered, true, true);
-  return array_to_all(data.slice, slice_size);
-}
-
 /**********************************************************************/
 /* increment a multi-index <n_1, n_2, ..., n_r> in which n_i runs over*/
 /* the range 0 <= n_i < nMax_i. return true if this is the increment  */
@@ -659,6 +524,10 @@ double *collapse_array(double *array, int *rank, size_t dims[3], direction dirs[
       reduced_dims[reduced_rank++] = dims[r];
     }
   }
+  if (reduced_rank==0) {
+    *rank = 0;
+    return array; // return array as is for singleton use case
+  }
   if (reduced_rank == full_rank) return array; // nothing to collapse
 
   /*--------------------------------------------------------------*/
@@ -670,11 +539,8 @@ double *collapse_array(double *array, int *rank, size_t dims[3], direction dirs[
   else if (full_rank == 3) {
     stride[0] = dims[1] * dims[2];
     stride[1] = dims[2];
-    if (reduced_stride[0] != 0)
-      reduced_stride[0] = reduced_dims[1];
-    else if (reduced_stride[1] != 0)
-      reduced_stride[1] = reduced_dims[1];
-    // else: two degenerate dimensions->reduced array is 1-diml, no strides needed
+    if (reduced_rank == 2)
+      reduced_stride[reduced_stride[0] != 0 ? 0 : 1] = reduced_dims[1];
   }
 
   /*--------------------------------------------------------------*/
@@ -708,25 +574,195 @@ cdouble *collapse_array(cdouble *array, int *rank, size_t dims[3], direction dir
   return (cdouble *)collapse_array((double *)array, rank, dims, dirs, where, 2);
 }
 
+/**********************************************************************/
+/* precisely one of fun, rfun, should be non-NULL                     */
+/**********************************************************************/
+void *fields::do_get_array_slice(const volume &where, std::vector<component> components,
+                                 field_function fun, field_rfunction rfun, void *fun_data,
+                                 void *vslice, double frequency, bool snap) {
+  am_now_working_on(FieldOutput);
+
+  /***************************************************************/
+  /* call get_array_slice_dimensions to get slice dimensions and */
+  /* partially initialze an array_slice_data struct              */
+  /***************************************************************/
+  size_t dims[3];
+  direction dirs[3];
+  array_slice_data data;
+  int rank = get_array_slice_dimensions(where, dims, dirs, false, snap, 0, &data);
+  size_t slice_size = data.slice_size;
+  bool complex_data = (rfun == 0);
+  int elem_size = complex_data ? 2 : 1;
+  void *vslice_uncollapsed;
+
+  vslice_uncollapsed = memset(new double[slice_size * elem_size], 0, slice_size * elem_size * sizeof(double));
+
+  data.vslice = vslice_uncollapsed;
+  data.snap = snap;
+  data.fun = fun;
+  data.rfun = rfun;
+  data.fun_data = fun_data;
+  data.components = components;
+  data.frequency = frequency;
+  int num_components = components.size();
+  data.cS = new component[num_components];
+  data.ph = new cdouble[num_components];
+  data.fields = new cdouble[num_components];
+  data.offsets = new ptrdiff_t[2 * num_components];
+  memset(data.offsets, 0, 2 * num_components * sizeof(ptrdiff_t));
+  data.empty_dim[0] = data.empty_dim[1] = data.empty_dim[2] = data.empty_dim[3] =
+      data.empty_dim[4] = false;
+  LOOP_OVER_DIRECTIONS(where.dim, d) { data.empty_dim[d] = where.in_direction(d) == 0; }
+
+  /* compute inverse-epsilon directions for computing Dielectric fields */
+  data.ninveps = 0;
+  bool needs_dielectric = false;
+  for (int i = 0; i < num_components; ++i)
+    if (components[i] == Dielectric) {
+      needs_dielectric = true;
+      break;
+    }
+  if (needs_dielectric) FOR_ELECTRIC_COMPONENTS(c) if (gv.has_field(c)) {
+      if (data.ninveps == 3) abort("more than 3 field components??");
+      data.inveps_cs[data.ninveps] = c;
+      data.inveps_ds[data.ninveps] = component_direction(c);
+      ++data.ninveps;
+    }
+
+  /* compute inverse-mu directions for computing Permeability fields */
+  data.ninvmu = 0;
+  bool needs_permeability = false;
+  for (int i = 0; i < num_components; ++i)
+    if (components[i] == Permeability) {
+      needs_permeability = true;
+      break;
+    }
+  if (needs_permeability) FOR_MAGNETIC_COMPONENTS(c) if (gv.has_field(c)) {
+      if (data.ninvmu == 3) abort("more than 3 field components??");
+      data.invmu_cs[data.ninvmu] = c;
+      data.invmu_ds[data.ninvmu] = component_direction(c);
+      ++data.ninvmu;
+    }
+
+  loop_in_chunks(get_array_slice_chunkloop, (void *)&data, where, Centered, true, snap);
+
+  if (!snap) {
+    double *slice = collapse_array((double *)vslice_uncollapsed, &rank, dims, dirs, where, elem_size);
+    rank = get_array_slice_dimensions(where, dims, dirs, true, false, 0, &data);
+    slice_size = data.slice_size;
+    vslice_uncollapsed = (double*) slice;
+  }
+  if (vslice) {
+    memcpy(vslice, vslice_uncollapsed, slice_size * elem_size * sizeof(double));
+    delete[] (double*) vslice_uncollapsed;
+  }
+  else
+    vslice = vslice_uncollapsed;
+
+  array_to_all((double *)vslice, elem_size * slice_size);
+
+  delete[] data.offsets;
+  delete[] data.fields;
+  delete[] data.ph;
+  delete[] data.cS;
+  finished_working();
+
+  return vslice;
+}
+
+/***************************************************************/
+/* entry points to get_array_slice                             */
+/***************************************************************/
+double *fields::get_array_slice(const volume &where, std::vector<component> components,
+                                field_rfunction rfun, void *fun_data, double *slice,
+                                double frequency, bool snap) {
+  return (double *)do_get_array_slice(where, components, 0, rfun, fun_data, (void *)slice,
+                                      frequency, snap);
+}
+
+cdouble *fields::get_complex_array_slice(const volume &where, std::vector<component> components,
+                                         field_function fun, void *fun_data, cdouble *slice,
+                                         double frequency, bool snap) {
+  return (cdouble *)do_get_array_slice(where, components, fun, 0, fun_data, (void *)slice,
+                                       frequency, snap);
+}
+
+double *fields::get_array_slice(const volume &where, component c, double *slice, double frequency,
+                                bool snap) {
+  std::vector<component> components(1);
+  components[0] = c;
+  return (double *)do_get_array_slice(where, components, 0, default_field_rfunc, 0, (void *)slice,
+                                      frequency, snap);
+}
+
+double *fields::get_array_slice(const volume &where, derived_component c, double *slice,
+                                double frequency, bool snap) {
+  int nfields;
+  component carray[12];
+  field_rfunction rfun = derived_component_func(c, gv, nfields, carray);
+  std::vector<component> cs(carray, carray + nfields);
+  return (double *)do_get_array_slice(where, cs, 0, rfun, &nfields, (void *)slice,
+                                      frequency, snap);
+}
+
+cdouble *fields::get_complex_array_slice(const volume &where, component c, cdouble *slice,
+                                         double frequency, bool snap) {
+  std::vector<component> components(1);
+  components[0] = c;
+  return (cdouble *)do_get_array_slice(where, components, default_field_func, 0, 0, (void *)slice,
+                                       frequency, snap);
+}
+
+cdouble *fields::get_source_slice(const volume &where, component source_slice_component,
+                                  cdouble *slice) {
+  size_t dims[3];
+  direction dirs[3];
+  vec min_max_loc[2];
+  int rank = get_array_slice_dimensions(where, dims, dirs, false, false, min_max_loc);
+  size_t slice_size = dims[0] * (rank >= 2 ? dims[1] : 1) * (rank == 3 ? dims[2] : 1);
+
+  source_slice_data data;
+  data.source_component = source_slice_component;
+  data.slice_imin = gv.round_vec(min_max_loc[0]);
+  data.slice_imax = gv.round_vec(min_max_loc[1]);
+  data.slice = new cdouble[slice_size];
+  if (!data.slice) abort("%s:%i: out of memory (%zu)", __FILE__, __LINE__, slice_size);
+
+  loop_in_chunks(get_source_slice_chunkloop, (void *)&data, where, Centered, true, false);
+
+  cdouble *slice_collapsed = collapse_array(data.slice, &rank, dims, dirs, where);
+  rank = get_array_slice_dimensions(where, dims, dirs, true, false);
+  slice_size = dims[0] * (rank >= 2 ? dims[1] : 1) * (rank == 3 ? dims[2] : 1);
+
+  if (slice) {
+    memcpy(slice, slice_collapsed, 2 * slice_size * sizeof(double));
+    delete[] (cdouble*) slice_collapsed;
+  }
+  else
+    slice = slice_collapsed;
+
+  array_to_all(slice, slice_size);
+
+  return slice;
+}
+
 /***************************************************************/
 /***************************************************************/
 /***************************************************************/
-std::vector<double> fields::get_array_metadata(const volume &where, bool collapse_empty_dimensions,
-                                               bool snap_empty_dimensions) {
+std::vector<double> fields::get_array_metadata(const volume &where) {
 
   /* get extremal corners of subgrid and array of weights, collapsed if necessary */
   size_t dims[3];
   direction dirs[3];
   vec min_max_loc[2]; // extremal points in subgrid
-  int rank = get_array_slice_dimensions(where, dims, dirs, false /*collapse_empty_dimensions*/,
-                                        snap_empty_dimensions, min_max_loc);
+  int rank = get_array_slice_dimensions(where, dims, dirs, true, false, min_max_loc);
+
   int full_rank = rank;
   direction full_dirs[3];
   for (int fr = 0; fr < rank; fr++)
     full_dirs[fr] = dirs[fr];
 
   double *weights = get_array_slice(where, NO_COMPONENT);
-  if (collapse_empty_dimensions) weights = collapse_array(weights, &rank, dims, dirs, where);
 
   /* get length and endpoints of x,y,z tics arrays */
   size_t nxyz[3] = {1, 1, 1};
@@ -734,7 +770,7 @@ std::vector<double> fields::get_array_metadata(const volume &where, bool collaps
   for (int fr = 0, rr = 0; fr < full_rank; fr++) {
     direction d = full_dirs[fr];
     int nd = d - X;
-    if (where.in_direction(d) == 0.0 && collapse_empty_dimensions) {
+    if (where.in_direction(d) == 0.0) {
       xyzmin[nd] = xyzmax[nd] = where.in_direction_min(d);
       nxyz[nd] = 1;
     }
@@ -756,6 +792,7 @@ std::vector<double> fields::get_array_metadata(const volume &where, bool collaps
   for (unsigned nw = 0; nw < (nxyz[0] * nxyz[1] * nxyz[2]); nw++)
     xyzw.push_back(weights[nw]);
 
+  delete[] weights;
   return xyzw;
 
 } // get_array_metadata
