@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2019 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2021 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -122,15 +122,62 @@ static void split_by_cost(std::vector<int> factors, grid_volume gvol,
 
 void structure::choose_chunkdivision(const grid_volume &thegv, int desired_num_chunks,
                                      const boundary_region &br, const symmetry &s) {
-  user_volume = thegv;
-  if (desired_num_chunks == 0) desired_num_chunks = count_processors();
+
   if (thegv.dim == Dcyl && thegv.get_origin().r() < 0) abort("r < 0 origins are not supported");
+
+  user_volume = thegv;
   gv = thegv;
   v = gv.surroundings();
   S = s;
   a = gv.a;
   dt = Courant / a;
 
+  // create the chunks:
+  std::vector<grid_volume> chunk_volumes = meep::choose_chunkdivision(gv, v, desired_num_chunks, s);
+
+  int my_num_chunks = chunk_volumes.size();
+
+  // initialize effort volumes
+  num_effort_volumes = 1;
+  effort_volumes = new grid_volume[num_effort_volumes];
+  effort_volumes[0] = gv;
+  effort = new double[num_effort_volumes];
+  effort[0] = 1.0;
+
+  // Next, add effort volumes for PML boundary regions:
+  br.apply(this);
+
+  // Break off PML regions into their own chunks
+  num_chunks = 0;
+  chunks = new structure_chunk_ptr[my_num_chunks * num_effort_volumes];
+  for (size_t i = 0, stop = chunk_volumes.size(); i < stop; ++i) {
+    const int proc = i * count_processors() / my_num_chunks;
+    for (int j = 0; j < num_effort_volumes; ++j) {
+      grid_volume vc;
+      if (chunk_volumes[i].intersect_with(effort_volumes[j], &vc)) {
+        chunks[num_chunks] = new structure_chunk(vc, v, Courant, proc);
+        br.apply(this, chunks[num_chunks++]);
+      }
+    }
+  }
+
+  check_chunks();
+
+  if (meep_geom::fragment_stats::resolution != 0) {
+    // Save cost of each chunk's grid_volume
+    for (int i = 0; i < num_chunks; ++i) {
+      chunks[i]->cost = chunks[i]->gv.get_cost();
+    }
+  }
+
+}
+
+std::vector<grid_volume> choose_chunkdivision(grid_volume &gv, volume &v, int desired_num_chunks,
+                                              const symmetry &S) {
+
+  if (desired_num_chunks == 0) desired_num_chunks = count_processors();
+  if (gv.dim == Dcyl && gv.get_origin().r() < 0) abort("r < 0 origins are not supported");
+
   // First, reduce overall grid_volume gv by symmetries:
   if (S.multiplicity() > 1) {
     bool break_this[3];
@@ -138,9 +185,9 @@ void structure::choose_chunkdivision(const grid_volume &thegv, int desired_num_c
       const direction d = (direction)dd;
       break_this[dd] = false;
       for (int n = 0; n < S.multiplicity(); n++)
-        if (has_direction(thegv.dim, d) &&
+        if (has_direction(gv.dim, d) &&
             (S.transform(d, n).d != d || S.transform(d, n).flipped)) {
-          if (thegv.num_direction(d) & 1 && !break_this[d] && verbosity > 0)
+          if (gv.num_direction(d) & 1 && !break_this[d] && verbosity > 0)
             master_printf("Padding %s to even number of grid points.\n", direction_name(d));
           break_this[dd] = true;
         }
@@ -164,15 +211,12 @@ void structure::choose_chunkdivision(const grid_volume &thegv, int desired_num_c
   }
 
   // initialize effort volumes
-  num_effort_volumes = 1;
-  effort_volumes = new grid_volume[num_effort_volumes];
+  int num_effort_volumes = 1;
+  grid_volume *effort_volumes = new grid_volume[num_effort_volumes];
   effort_volumes[0] = gv;
-  effort = new double[num_effort_volumes];
+  double *effort = new double[num_effort_volumes];
   effort[0] = 1.0;
 
-  // Next, add effort volumes for PML boundary regions:
-  br.apply(this);
-
   std::vector<int> prime_factors = get_prime_factors(desired_num_chunks);
 
   // We may have to use a different number of chunks than the user requested
@@ -180,57 +224,35 @@ void structure::choose_chunkdivision(const grid_volume &thegv, int desired_num_c
   for (size_t i = 0, stop = prime_factors.size(); i < stop; ++i)
     adjusted_num_chunks *= prime_factors[i];
 
+  int my_num_chunks =
+      meep_geom::fragment_stats::split_chunks_evenly ? desired_num_chunks : adjusted_num_chunks;
+
   // Finally, create the chunks:
-  num_chunks = 0;
-  chunks = new structure_chunk_ptr[adjusted_num_chunks * num_effort_volumes];
   std::vector<grid_volume> chunk_volumes;
 
-  bool by_cost = false;
   if (meep_geom::fragment_stats::resolution == 0 ||
-      meep_geom::fragment_stats::has_non_medium_material() ||
       meep_geom::fragment_stats::split_chunks_evenly) {
-    if (verbosity > 0 && adjusted_num_chunks > 1)
-      master_printf("Splitting into %d chunks evenly\n", adjusted_num_chunks);
-    for (int i = 0; i < adjusted_num_chunks; i++) {
+    if (verbosity > 0 && my_num_chunks > 1)
+      master_printf("Splitting into %d chunks evenly\n", my_num_chunks);
+    for (int i = 0; i < my_num_chunks; i++) {
       grid_volume vi =
-          gv.split_by_effort(adjusted_num_chunks, i, num_effort_volumes, effort_volumes, effort);
+          gv.split_by_effort(my_num_chunks, i, num_effort_volumes, effort_volumes, effort);
       chunk_volumes.push_back(vi);
     }
   }
   else {
-    if (verbosity > 0 && adjusted_num_chunks > 1)
-      master_printf("Splitting into %d chunks by cost\n", adjusted_num_chunks);
+    if (verbosity > 0 && my_num_chunks > 1)
+      master_printf("Splitting into %d chunks by cost\n", my_num_chunks);
     split_by_cost(prime_factors, gv, chunk_volumes);
-    by_cost = true;
   }
 
-  // Break off PML regions into their own chunks
-  for (size_t i = 0, stop = chunk_volumes.size(); i < stop; ++i) {
-    const int proc = i * count_processors() / adjusted_num_chunks;
-    for (int j = 0; j < num_effort_volumes; ++j) {
-      grid_volume vc;
-      if (chunk_volumes[i].intersect_with(effort_volumes[j], &vc)) {
-        chunks[num_chunks] = new structure_chunk(vc, v, Courant, proc);
-        br.apply(this, chunks[num_chunks++]);
-      }
-    }
-  }
-  check_chunks();
-
-  if (by_cost) {
-    // Save cost of each chunk's grid_volume
-    for (int i = 0; i < num_chunks; ++i) {
-      chunks[i]->cost = chunks[i]->gv.get_cost();
-    }
-  }
+  return chunk_volumes;
 }
 
 double structure::estimated_cost(int process) {
   double proc_cost = 0;
   for (int i = 0; i < num_chunks; i++) {
-    if (chunks[i]->n_proc() == process) {
-      proc_cost += chunks[i]->cost;
-    }
+    if (chunks[i]->n_proc() == process) { proc_cost += chunks[i]->cost; }
   }
   return proc_cost;
 }
@@ -456,6 +478,7 @@ void structure::set_epsilon(material_function &eps, bool use_anisotropic_averagi
                             int maxeval) {
   double tstart = wall_time();
   FOR_ELECTRIC_COMPONENTS(c) { set_chi1inv(c, eps, use_anisotropic_averaging, tol, maxeval); }
+  all_wait(); // sync so that timing results are accurate
   if (verbosity > 0) master_printf("time for set_epsilon = %g s\n", wall_time() - tstart);
 }
 
@@ -469,6 +492,7 @@ void structure::set_mu(material_function &m, bool use_anisotropic_averaging, dou
                        int maxeval) {
   double tstart = wall_time();
   FOR_MAGNETIC_COMPONENTS(c) { set_chi1inv(c, m, use_anisotropic_averaging, tol, maxeval); }
+  all_wait(); // sync so that timing results are accurate
   if (verbosity > 0) master_printf("time for set_mu = %g s\n", wall_time() - tstart);
 }
 
@@ -484,6 +508,7 @@ void structure::set_conductivity(component c, material_function &C) {
   changing_chunks();
   for (int i = 0; i < num_chunks; i++)
     if (chunks[i]->is_mine()) chunks[i]->set_conductivity(c, C);
+  all_wait(); // sync so that timing results are accurate
   if (verbosity > 0) master_printf("time for set_conductivity = %g s\n", wall_time() - tstart);
 }
 
@@ -817,7 +842,7 @@ structure_chunk::structure_chunk(const structure_chunk *o) : v(o->v) {
   }
   condinv_stale = o->condinv_stale;
   // Allocate the PML conductivity arrays:
-  FOR_DIRECTIONS(d) {
+  for (int d = 0; d < 6; ++d) {
     sig[d] = NULL;
     kap[d] = NULL;
     siginv[d] = NULL;
@@ -974,7 +999,7 @@ structure_chunk::structure_chunk(const grid_volume &thegv, const volume &vol_lim
     condinv[c][d] = NULL;
   }
   condinv_stale = false;
-  FOR_DIRECTIONS(d) {
+  for (int d = 0; d < 6; ++d) {
     sig[d] = NULL;
     kap[d] = NULL;
     siginv[d] = NULL;
